﻿using System;

namespace Ascon.AppPlugin
{
    class AppCacheWrapper : IAppCache
    {
        private readonly IPluginAppCache appCache;

        public AppCacheWrapper(IPluginAppCache appCache)
        {
            if (appCache == null)
                throw new ArgumentNullException("appCache");
            this.appCache = appCache;
        }

        private void ApiCheck(bool res)
        {
            if (!res)
                throw new Exception(appCache.GetLastError());
        }

        public string GetDBName()
        {
            return appCache.GetDBName();
        }

        public string GetUserName()
        {
            return appCache.GetUserName();
        }

        public string GetUserFullName()
        {
            return appCache.GetUserFullName();
        }

        public IPluginDataSet GetTypes()
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetTypes(out res));
            return res;
        }

        public IPluginDataSet GetStates()
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetStates(out res));
            return res;    
        }

        public IPluginDataSet GetDBProperties()
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetDBProperties(out res));
            return res;               
        }

        public IPluginDataSet GetLinks()
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetLinks(out res));
            return res;               
        }

        public IPluginDataSet GetAttributeList()
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetAttributeList(out res));
            return res;               
        }

        public IPluginDataSet GetUserList()
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetUserList(out res));
            return res;               
        }

        public IPluginDataSet GetInfoAboutType(string TypeName, int inMode)
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetInfoAboutType(TypeName, inMode, out res));
            return res;               
        }

        public IPluginDataSet GetLinkAttrForTypes(string ParentType, string TypeObj, string LinkType)
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetLinkAttrForTypes(ParentType, TypeObj, LinkType, out res));
            return res;               
        }

        public IPluginDataSet GetInfoAboutCard(string CardName, int inMode)
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetInfoAboutCard(CardName, inMode, out res));
            return res;              
        }

        public IPluginDataSet GetInfoAboutAttribute(string AttrName, int inMode)
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetInfoAboutAttribute(AttrName, inMode, out res));
            return res;               
        }

        public IPluginDataSet GetInfoAboutCurrentBase(int inMode)
        {
            IPluginDataSet res;
            ApiCheck(appCache.GetInfoAboutCurrentBase(inMode, out res));
            return res;               
        }

        public string GetDefaultLink(string TypeName, string ParentTypeName)
        {
            string res;
            ApiCheck(appCache.GetDefaultLink(TypeName, ParentTypeName, out res));
            return res;   
        }

        public string GetDefaultState(string TypeName)
        {
            string res;
            ApiCheck(appCache.GetDefaultState(TypeName, out res));
            return res;  
        }

        public bool IsDocument(string TypeObj)
        {
            bool res;
            ApiCheck(appCache.IsDocument(TypeObj, out res));
            return res;  
        }

        public bool IsDocument(int TypeId)
        {
            bool res;
            ApiCheck(appCache.IsDocument(TypeId, out res));
            return res;  
        }

        public string GetTypeNameById(int Id)
        {
            string res;
            ApiCheck(appCache.GetTypeNameById(Id, out res));
            return res;  
        }

        public string GetStateNameById(int Id)
        {
            string res;
            ApiCheck(appCache.GetStateNameById(Id, out res));
            return res;  
        }

        public string GetLinkNameById(int Id)
        {
            string res;
            ApiCheck(appCache.GetLinkNameById(Id, out res));
            return res;  
        }

        public IPluginPDMVersions LoadPDMVersionsByIds(string AIds)
        {
            IPluginPDMVersions res;
            ApiCheck(appCache.LoadPDMVersionsByIds(AIds, out res));
            return res;  
        }
		
		public IPluginDataSet GetAutoOperationList()
		{
			IPluginDataSet res;
			ApiCheck(appCache.GetAutoOperationList(out res));
			return res;
		}
    }
}
