unit PluginIntfWrapper;

interface

uses
  SysUtils,
  ComObj,
  ActiveX,

  PluginIntf;

type
  EApiException = class(Exception);

  TAppCache = class(TObject)
  private
    FAppCache: IPluginAppCache;
    procedure ApiCheck(AResult: WordBool);
  public
    constructor Create(AAppCache: IPluginAppCache);

    function GetDBName: WideString;
    function GetUserName: WideString;
    function GetUserFullName: WideString;

    function GetTypes: IPluginDataSet;
    function GetStates: IPluginDataSet;
    function GetDBProperties: IPluginDataSet;
    function GetLinks: IPluginDataSet;
    function GetAttributeList: IPluginDataSet;
    function GetUserList: IPluginDataSet;
    function GetInfoAboutType(const TypeName: WideString; inMode: Integer): IPluginDataSet;
    /// <summary>  GetLinkAttrForTypes.</summary>
    /// <returns>      .</returns>
    function GetLinkAttrForTypes(const ParentType, TypeObj, LinkType: WideString): IPluginDataSet;
    function GetInfoAboutCard(const CardName: WideString; inMode: Integer): IPluginDataSet;
    function GetInfoAboutAttribute(const AttrName: WideString; inMode: Integer): IPluginDataSet;
    function GetInfoAboutCurrentBase(inMode: Integer): IPluginDataSet;

    function GetDefaultLink(const TypeName, ParentTypeName: WideString): WideString;
    function GetDefaultState(const TypeName: WideString): WideString;

    /// <summary>    .</summary>
    /// <param name="TypeObj"> .</param>
    /// <returns>True -    .</returns>
    function IsDocument(const TypeObj: WideString): WordBool; overload;
    /// <summary>    .</summary>
    /// <param name="TypeId"> .</param>
    /// <returns>True -    .</returns>
    function IsDocument(TypeId: Integer): WordBool; overload;
    function GetTypeNameById(Id: Integer): WideString;
    function GetStateNameById(Id: Integer): WideString;
    function GetLinkNameById(Id: Integer): WideString;
    function LoadPDMVersionsByIds(AIds: WideString): IPluginPDMVersions;
    function GetAutoOperationList(): IPluginDataSet;
  end;


  TAppMain = class(TObject)
  private
    FAppMain: IPluginAppMain;
    procedure ApiCheck(AResult: WordBool);
  public
    constructor Create(AAppMain: IPluginAppMain);

    // API-    
    procedure Client_CreateFile(TargetFile: WideString; IdCheckout, IdVersion: Integer;
      FileName: WideString);
    procedure Client_DeleteFile(IdCheckout, IdVersion: Integer;
      FileName: WideString);
    procedure Client_UpdateFile(TargetFile: WideString; IdCheckout, IdVersion: Integer;
      FileName: WideString);
    function Client_GetFilePath(IdVersion: Integer; stFileName: WideString): WideString;
    function Client_GetFile(IdCheckout: Integer; stTypeObj, stProduct, stVersion,
      stFileName, stLocalName: WideString; inSize, inCRC: Integer;
      dtCreate, dtModify: TDateTime; RevertLocalChanges: WordBool): WideString;
    function Client_GetFileById(IdCheckout, IdVersion: Integer;
      stFileName, stLocalName: WideString; inSize, inCRC: Integer;
      dtCreate, dtModify: TDateTime; RevertLocalChanges: WordBool): WideString;

    // API-
    function CheckUniqueName(stTypeName, stProductName: WideString): IPluginDataSet;
    function ExistsObject(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber: WideString; inIdVersion: Integer): Integer;
    function FindObjects(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber, stStateName, stAttrCondition, stProjects,
      stLinkType: WideString): IPluginDataSet;
    function FindObjectsInContext2(IdCheckOut: Integer;
      stContext, stSearchXML: WideString; inParams: Integer): IPluginDataSet;
    function GetAllLinkedObjects(IdCheckOut: Integer; Ids: WideString;
      inParams: Integer): IPluginDataSet;
    function GetAttributeList: IPluginDataSet;
    function GetAttributesValues2(stIds, stAttrIds: WideString): IPluginDataSet;
    function GetAttrImageValueById(IdCheckOut, inIdVersion: Integer;
      stAttrName: WideString; inIdLink: Integer): IPluginDataSet;
    function GetInfoAboutAttribute(stAttrName: WideString; inMode: Integer): IPluginDataSet;
    function GetInfoAboutCard(stCardName: WideString; inMode: Integer): IPluginDataSet;
    function GetInfoAboutCurrentBase(inMode: Integer): IPluginDataSet;
    function GetInfoAboutCurrentUser: IPluginDataSet;
    function GetInfoAboutLink(IdCheckOut, inIdLink, inMode: Integer): IPluginDataSet;
    function GetInfoAboutType(stTypeName: WideString; inMode: Integer): IPluginDataSet;
    function GetInfoAboutUserSet(stUserSet, stDBName: WideString;
      inMode: Integer): IPluginDataSet;
    function GetInfoAboutVersion(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber: WideString; inIdVersion, inMode: Integer): IPluginDataSet;
    function GetInfoAboutVersionsFiles(IdCheckOut: Integer; stIds: WideString): IPluginDataSet;
    function GetInfoAboutVersionsPrivileges(stIds: WideString): IPluginDataSet;
    function GetLinkAttrForTypes(stParentType, stChildType,
      stLinkType: WideString): IPluginDataSet;
    function GetLinkAttributes(IdCheckOut, inIdLink: Integer): IPluginDataSet;
    function GetLinkedFast(IdCheckOut, inIdVersion: Integer; stLinkType: WideString;
      boInverse: WordBool): IPluginDataSet;
    function GetLinkedFast2(IdCheckOut, inIdVersion: Integer; stLinkType: WideString;
      boInverse: WordBool): IPluginDataSet;
    function GetLinkedObjects2(IdCheckOut, inIdVersion: Integer;
      stLinkType: WideString; boInverse, boFullLink, boGroupByProduct: WordBool): IPluginDataSet;
    function GetLinkedObjectsAndFiles(IdCheckOut: Integer;
      stTypeName, stProductName, stVersionNumber, stLinkType: WideString;
      boFullLink, boViewOnlyDocuments: WordBool): IPluginDataSet;
    function GetLinkedObjectsEx(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber: WideString; inIdVersion: Integer; stLinkType: WideString;
      boInverse, boFullLink, boGroupByProduct: WordBool): IPluginDataSet;
    function GetLinks(IdCheckOut, inIdVersion1, inIdVersion2: Integer): IPluginDataSet;
    function GetLObjs(IdCheckOut, inIdVersion: Integer; boInverse: WordBool): IPluginDataSet;
    function GetLockedObjects(IdCheckOut, inParams: Integer): IPluginDataSet;
    function GetPrivilegesForObjects(IdCheckOut: Integer; stIds: WideString;
      Mode: Integer): IPluginDataSet;
    function GetProjectList2(IdCheckOut: Integer; inParams: Integer): IPluginDataSet;
    function GetProjectListEx(IdCheckOut: Integer; boWithAttrs: WordBool): IPluginDataSet;
    function GetPropObjects2(IdCheckOut: Integer; stObjects: WideString;
      inParams: Integer): IPluginDataSet;
    function GetPropObjects3(IdCheckOut: Integer; Ids, stAttrIds: WideString): IPluginDataSet;
    function GetStateList: IPluginDataSet;
    function GetTree(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber: WideString; inIdVersion: Integer; stLinkType: WideString;
      boWithAttrs: WordBool): IPluginDataSet;
    function GetTree2(IdCheckOut, inIdVersion: Integer; stLinkType: WideString;
      inParams: Integer): IPluginDataSet;
    function GetTypeList: IPluginDataSet;
    function GetUserGroupList: IPluginDataSet;
    function GetUserList: IPluginDataSet;
    function GetVersionList(IdCheckOut: Integer; stTypeName,
      stProductName: WideString): IPluginDataSet;
    function GetVersionList2(IdCheckOut: Integer; inIdType: Integer;
      stProductName: WideString): IPluginDataSet;
    function InsertObject(IdCheckOut: Integer; stParentType, stParentProduct,
      stParentVersion, stChildType, stChildProduct, stChildVersion, stLinkType,
      stState: WideString; boKeyInsert: WordBool): Integer;
    procedure IsProject(stTypeName, stProductName: WideString;
      boIsProject: WordBool);
    procedure KillVersionById(IdCheckOut, inIdVersion: Integer);
    function KillVersions(IdCheckOut: Integer; stIds: WideString;
      inParams: Integer): IPluginDataSet;
    function NewObject(IdCheckOut: Integer; stTypeName, stStateName,
      stProductName: WideString; inProject: Integer): Integer;
    procedure NewUserSet(stUserSet, stDBName, stParentUserSet: WideString);
    function OpenUserSet2(stUserSet: WideString; inParams: Integer): IPluginDataSet;
    procedure UpdateStateOnObjectById(IdCheckOut, inIdVersion: Integer;
      stStateName: WideString);
    procedure UpFindPattern(stFindPattern, stUserSet, stDBName, stTypePattern,
      stProductPattern, stVersionPattern, stStatePattern, stAttrCondition,
      stProjectPattern: WideString; nIdPattern: Integer;
      boActive, boDel: WordBool);
    procedure UpGrantOnVersion(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber, stOrgUnitName: WideString; inPriveleges: Integer;
      dtFinishTime: TDateTime; boDel: WordBool);
    function UpLink(IdCheckOut: Integer; stParentType, stParentProduct,
      stParentVersion, stChildType, stChildProduct, stChildVersion: WideString;
      inIdLink: Integer; {reMinQuantity, reMaxQuantity: Variant;} stIdUnit: WideString;
      boDel: WordBool; stLinkType: WideString): Integer;
    function UpObject(IdCheckOut: Integer; stTypeName, stProductName,
      stNewProductName: WideString): Integer;
    function CheckOut(stTypeName, stProductName, stVersionNumber: WideString;
      inMode: Integer): Integer;
    procedure CheckIn(IdCheckOut: Integer);
    procedure CancelCheckOut(IdCheckOut: Integer);
    //  ,    vaAttrValue
    procedure UpAttrValue(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber, stAttrName: WideString; vaAttrValue: Integer;
      stIdUnit: WideString; boDel: WordBool); overload;
    procedure UpAttrValue(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber, stAttrName: WideString; vaAttrValue: WideString;
      stIdUnit: WideString; boDel: WordBool); overload;
    procedure UpAttrValue(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber, stAttrName: WideString; vaAttrValue: TDateTime;
      stIdUnit: WideString; boDel: WordBool); overload;
    procedure UpAttrValue(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber, stAttrName: WideString; vaAttrValue: Double;
      stIdUnit: WideString; boDel: WordBool); overload;
    //  ,    vaAttrValue
    procedure UpAttrValueById(IdCheckOut, inIdVersion: Integer;
      stAttrName: WideString; vaAttrValue: Integer; stIdUnit: WideString;
      boDel: WordBool); overload;
    procedure UpAttrValueById(IdCheckOut, inIdVersion: Integer;
      stAttrName: WideString; vaAttrValue: WideString; stIdUnit: WideString;
      boDel: WordBool); overload;
    procedure UpAttrValueById(IdCheckOut, inIdVersion: Integer;
      stAttrName: WideString; vaAttrValue: TDateTime; stIdUnit: WideString;
      boDel: WordBool); overload;
    procedure UpAttrValueById(IdCheckOut, inIdVersion: Integer;
      stAttrName: WideString; vaAttrValue: Double; stIdUnit: WideString;
      boDel: WordBool); overload;
    //  ,    vaAttrValue
    procedure UpLinkAttrValue(IdCheckOut, inIdLink: Integer; stAttrName: WideString;
      vaAttrValue: Integer; stIdUnit: WideString; boDel: WordBool); overload;
    procedure UpLinkAttrValue(IdCheckOut, inIdLink: Integer; stAttrName: WideString;
      vaAttrValue: WideString; stIdUnit: WideString; boDel: WordBool); overload;
    procedure UpLinkAttrValue(IdCheckOut, inIdLink: Integer; stAttrName: WideString;
      vaAttrValue: TDateTime; stIdUnit: WideString; boDel: WordBool); overload;
    procedure UpLinkAttrValue(IdCheckOut, inIdLink: Integer; stAttrName: WideString;
      vaAttrValue: Double; stIdUnit: WideString; boDel: WordBool); overload;
    function GetDBList: WideString;

    // API- -------------------------------------------------------------
    function GetSignatures(AIdCheckout, AVersionId: Integer): IPluginDataSet;
    function GetAnnotations(AIdCheckout, AVersionId: Integer): IPluginDataSet;

    procedure SetUserSetPattern(userSetName, dbName, xmlPattern: WideString);
    procedure UpUserSet(stUserSet, stDBName, stNewUserSet, stGroupName: WideString;
      inGroupOperation: Integer; boDel: WordBool);
    //--------------------------------------------------------------------------

    function GetReport(IdCheckOut: Integer; ReportName, Params: WideString): IPluginDataSet;

    /// <summary>    .</summary>
    /// <param name="IdCheckOut"> .</param>
    /// <param name="stTypeName"> .</param>
    /// <param name="stProductName"> .</param>
    /// <param name="stVersionNumber"> .</param>
    /// <param name="stStateName">  .</param>
    /// <param name="vaFileList"> ,      . 
    ///      ","  "|".   2- :
    ///  'file1.png, 5434\, file1.png, -5555\|file1.rtf, , file1.rtf, -55555\'
    ///      :
    ///  'file1.png, 5434\, file1.png, -5555\'
    ///             .
    /// </param>
    /// <param name="inGroup">    .</param>
    /// <param name="inParams">.</param>
    /// <returns> out res    .</returns>
    function NewVersionEx(IdCheckOut: Integer; stTypeName, stProductName,
      stVersionNumber, stStateName: WideString; vaFileList: WideString; inGroup,
      inParams: Integer): Integer;

    /// <summary>        TargetFile,
    ///  .
    /// !  Loodsman_ExtractFile    
    ///        Client_GetFileById
    /// </summary>
    procedure Loodsman_ExtractFile(IdCheckout: Integer; TypeObj, Product,
      Version: WideString; IdVersion: Integer;
      stFileName, stLocalName: WideString; inSize, inCRC: Integer; dtCreate, dtModify: TDateTime;
      TargetFile: WideString);

    procedure PrintFileToXPS(FileName, XPSFileName: WideString);

    procedure ClearTempUserDir;
    procedure SignAs(const XPSFileName, Login: WideString);
    //   
    (*
    function GrantRightsForObjects(objects: Variant): IPluginDataSet;
    /// <summary>      feature ,
    ///         ,  
    ///  generic .</summary>
    function CheckOutLicense(const hostId: WideString; feature:
      Integer; const description: WideString; options: Integer);
    *)
  end;


  TAppWorkFlow = class(TObject)
  private
    FAppWorkFlow: IPluginAppWorkFlow;
    procedure ApiCheck(AResult: WordBool);
  public
    constructor Create(AAppWorkFlow: IPluginAppWorkFlow);

    function CreateTask(Subj, Text: WideString; IdVersion,
      IdRole, IdUser: Integer; DatePlan: TDateTime; CriticalTerm: Integer;
      CheckResult: WordBool; Kind: TBProcessKindExp): Integer;
    procedure StartTask(IdRoute, IdRole, IdUser: Integer;
      Subj: WideString; DatePlan: TDateTime);
    function CreateTaskByDialog: Integer;

    // API- WorkFlow
    function GetIncomingCorrespondence(inRetMode: Integer): IPluginDataSet;
    function GetOutcomingCorrespondence(inRetMode: Integer): IPluginDataSet;
    function GetCorrespondenceList(inID_Route: Integer): IPluginDataSet;
    procedure DelLetter(inIDLetter: Integer);
    function GetUserList: IPluginDataSet;
    function GetRoleTree(inMode: Integer): IPluginDataSet;
    function GetProcessListByObject(stDBName: WideString; inPdm_Id: Integer): IPluginDataSet;
    function GetProcessListByObjectEx(inPdm_Id: Integer; stDBName: WideString): IPluginDataSet;
    function GetInitialProcessList: IPluginDataSet;
    function GetInfoAboutCurrentUser: IPluginDataSet;
    function GetInfoAboutRoute(inID_Route: Integer): IPluginDataSet;
    function GetRoleListForUser(inID_User: Integer): IPluginDataSet;
    function GetTaskListForUser(inIdUser: Integer; inUserRoleMask,
      inProcessStateMask, inStageStateMode: Integer): IPluginDataSet;
    function CreateNewProcess(stDBName: WideString; inID_PDM_OBJECT: Integer;
      stRouteName: WideString): Integer;
    procedure DeleteProcess(inID_Route: Integer);
    procedure InsUpdStageIntoBP(stTask: WideString;
      inCriticalTerm: Integer; dtDatePlan: TDateTime; inCanControl, inActive,
      inIdRole, inIdUser, inIdBProc, inCanTransform: Integer; out inIdStage,
      inTimeConstraint, inFromBPStart: Integer);
    procedure AddRouteItemsSeq(inIdFirst, inIdSecond, inType: Integer);
    procedure AddRouteItemsRetSeq(inIdFirst, inIdSecond: Integer);
    function GetRoleListIntoProcess(inID_Route: Integer): IPluginDataSet;
    procedure SetStateOfProcess(inID_Route, inStateBP: Integer);
    procedure InsUpdUser(stUserName, stDescr, stEMail: WideString; inIsAdmin: Integer;
      out vaIdUser: Integer);
    procedure SetTaskState(inID_Route, inID_Operation, inStateTask: Integer;
      stMessageBody: WideString);
    procedure SetItemState(inIdItem, inState, inAddSignature: Integer);
    procedure SetItemStateInternal(inIdItem, inState: Integer);
    procedure AddBPDocument(inIdRoute, inIdDoc, inIdPar: Integer);
    procedure DelBPDocument(inIdRoute, inIdDoc: Integer);
    function GetFirstItems(inIdRoute: Integer): IPluginDataSet;
    function GetNextItems(inIdItem: Integer): IPluginDataSet;
    function rep_ExecutedJobs(inID_Parent: Integer): IPluginDataSet;
    function GetJobsListIntoProcess(inID_Route: Integer): IPluginDataSet;
    function GetAttachDocumentListIntoProcess(inIdRoute: Integer): IPluginDataSet;
    procedure StartNextStage(inID_Route, inIdItem, inDirect: Integer);
    procedure RenameProcess(inID_Route: Integer; stNewName: WideString);
    procedure SetStageExecutor(inIdStage, inIdUser, inIdRole: Integer);
    procedure SetStageTerm(inIdStage, inTimeLimit: Integer;
      vaPlanEnd: TDateTime; inTimeConstraint, inFromBPStart: Integer);
    procedure SetStageTask(inIdStage: Integer; stTask: WideString);
    procedure SetBP_PDMObject(inIdRoute, inIdPDMObject: Integer);
    function GetTaskListForObject(inIdVersion, inProcessStateMask,
      inStageStateMode: Integer): IPluginDataSet;
    function GetInfoAboutStage(inIdStage: Integer): IPluginDataSet;
    function SendLetterByBPToUsers(inIdRoute: Integer; inIdAURole: Integer;
      vaRecipientList: IPluginDataSet; stSubj, stBody: WideString): Integer;

    procedure GetVariable(inIdRoute: Integer; stName: WideString; out res: Integer); overload;
    procedure GetVariable(inIdRoute: Integer; stName: WideString; out res: WideString); overload;
    procedure GetVariable(inIdRoute: Integer; stName: WideString; out res: TDateTime); overload;
    procedure GetVariable(inIdRoute: Integer; stName: WideString; out res: Double); overload;
    procedure SetVariable(inIdRoute: Integer; stName: WideString; Value: Integer; ReadOnlyForChild: WordBool); overload;
    procedure SetVariable(inIdRoute: Integer; stName: WideString; Value: WideString; ReadOnlyForChild: WordBool); overload;
    procedure SetVariable(inIdRoute: Integer; stName: WideString; Value: TDateTime; ReadOnlyForChild: WordBool); overload;
    procedure SetVariable(inIdRoute: Integer; stName: WideString; Value: Double; ReadOnlyForChild: WordBool); overload;

    function SendLetterByBPToOneUser(inIdRoute, inIdAURole, inIdRecRole, inIdRecUser: Integer;
        stSubj, stBody: WideString): Integer;

    function GetTypicalProcessList(inMode: Integer): IPluginDataSet;
  end;


  TAppConfigurator = class(TObject)
  private
    FAppConfigurator: IPluginAppConfigurator;
    procedure ApiCheck(AResult: WordBool);
  public
    constructor Create(AAppConfigurator: IPluginAppConfigurator);

    function GetSortCase(stProfile, stType, stLink: WideString;
      inIndex: Integer): IPluginDataSet;
    function GetSortTypes(inIdCase: Integer): IPluginDataSet;
    function GetSortTree(inIdCase: Integer): IPluginDataSet;
    function GetSortTable(inIdCase: Integer): IPluginDataSet;
  end;

  TAppProvider = class(TObject)
  private
    FAppProvider: IPluginAppProvider;
    FAppMain: TAppMain;
    FAppWorkFlow: TAppWorkFlow;
    FAppCache: TAppCache;
    FAppConfigurator: TAppConfigurator;
    procedure ApiCheck(AResult: WordBool);
  public
    constructor Create(AAppProvider: IPluginAppProvider);
    destructor Destroy; override;

    function GetAppTitle: WideString;
    function GetAppHandle: THandle;

    /// <summary>    .</summary>
    /// <returns>  .</returns>
    function GetSelectedObjects: IPluginPDMVersions;
    /// <summary>   .</summary>
    /// <param name="IdVersion"> .</param>
    procedure RefreshVersion(IdVersion: Integer);
    /// <summary>   .</summary>
    /// <param name="IdVersion"> .</param>
    /// <returns>  /   .</returns>
    procedure SelectObject(IdVersion: Integer);
    procedure SelectTask(IdProcess: Integer; TasksFilter: TTasksFilterExp);
    procedure SelectMail(IdLetter: Integer; Filter: TMailFilterExp);
    function GetSelectedTasks: IPluginTasks;
    function GetAttachFromDlg(out Versions: IPluginPDMVersions): TModalResultExp;
    function GetUsersFromDlg(ADateVisible, ASignRoleVisible,
      AMultiselect, ACan_I_Am: Boolean; ASelectedUsers: IPluginUsers;
      out Users: IPluginUsers): TModalResultExp;
    procedure RefreshTasks(ASelectedRouteId: Integer);

    property AppMain: TAppMain read FAppMain;
    property AppWorkFlow: TAppWorkFlow read FAppWorkFlow;
    property AppCache: TAppCache read FAppCache;
    property AppConfigurator: TAppConfigurator read FAppConfigurator;
  end;


implementation


{ TAppCache }

constructor TAppCache.Create(AAppCache: IPluginAppCache);
begin
  FAppCache := AAppCache;
end;

procedure TAppCache.ApiCheck(AResult: WordBool);
begin
  if not AResult then
    raise EApiException.Create(FAppCache.GetLastError);
end;

function TAppCache.GetAttributeList: IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetAttributeList(Result)
    );
end;

function TAppCache.GetAutoOperationList: IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetAutoOperationList(Result)
    );
end;

function TAppCache.GetDBName: WideString;
begin
  Result := FAppCache.GetDBName
end;

function TAppCache.GetDBProperties: IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetDBProperties(Result)
    );
end;

function TAppCache.GetDefaultLink(const TypeName,
  ParentTypeName: WideString): WideString;
begin
  ApiCheck(
    FAppCache.GetDefaultLink(TypeName, ParentTypeName, Result)
    );
end;

function TAppCache.GetDefaultState(const TypeName: WideString): WideString;
begin
  ApiCheck(
    FAppCache.GetDefaultState(TypeName, Result)
    );
end;

function TAppCache.GetInfoAboutAttribute(const AttrName: WideString;
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetInfoAboutAttribute(AttrName, inMode, Result)
    );
end;

function TAppCache.GetInfoAboutCard(const CardName: WideString;
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetInfoAboutCard(CardName, inMode, Result)
    );
end;

function TAppCache.GetInfoAboutCurrentBase(inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetInfoAboutCurrentBase(inMode, Result)
    );
end;

function TAppCache.GetInfoAboutType(const TypeName: WideString;
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetInfoAboutType(TypeName, inMode, Result)
    );
end;

function TAppCache.GetLinkAttrForTypes(const ParentType, TypeObj,
  LinkType: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetLinkAttrForTypes(ParentType, TypeObj, LinkType, Result)
    );
end;

function TAppCache.GetLinkNameById(Id: Integer): WideString;
begin
  ApiCheck(
    FAppCache.GetLinkNameById(Id, Result)
    );
end;

function TAppCache.GetLinks: IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetLinks(Result)
    );
end;

function TAppCache.GetStateNameById(Id: Integer): WideString;
begin
  ApiCheck(
    FAppCache.GetStateNameById(Id, Result)
    );
end;

function TAppCache.GetStates: IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetStates(Result)
    );
end;

function TAppCache.GetTypeNameById(Id: Integer): WideString;
begin
  ApiCheck(
    FAppCache.GetTypeNameById(Id, Result)
    );
end;

function TAppCache.GetTypes: IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetTypes(Result)
    );
end;

function TAppCache.GetUserFullName: WideString;
begin
  Result := FAppCache.GetUserFullName;
end;

function TAppCache.GetUserList: IPluginDataSet;
begin
  ApiCheck(
    FAppCache.GetUserList(Result)
    );
end;

function TAppCache.GetUserName: WideString;
begin
  Result := FAppCache.GetUserName;
end;

function TAppCache.IsDocument(TypeId: Integer): WordBool;
begin
  ApiCheck(
    FAppCache.IsDocument(TypeId, Result)
    );
end;

function TAppCache.LoadPDMVersionsByIds(AIds: WideString): IPluginPDMVersions;
begin
  ApiCheck(
      FAppCache.LoadPDMVersionsByIds(AIds, Result)
      );
end;

function TAppCache.IsDocument(const TypeObj: WideString): WordBool;
begin
  ApiCheck(
    FAppCache.IsDocument(TypeObj, Result)
    );
end;



{ TAppMain }

constructor TAppMain.Create(AAppMain: IPluginAppMain);
begin
  FAppMain := AAppMain;
end;

procedure TAppMain.ApiCheck(AResult: WordBool);
begin
  if not AResult then
    raise EApiException.Create(FAppMain.GetLastError);
end;

function TAppMain.ExistsObject(IdCheckOut: Integer; stTypeName, stProductName,
  stVersionNumber: WideString; inIdVersion: Integer): Integer;
begin
  ApiCheck(
    FAppMain.ExistsObject(IdCheckOut, stTypeName, stProductName, stVersionNumber,
                 inIdVersion, Result)
    );
end;

function TAppMain.FindObjects(IdCheckOut: Integer; stTypeName, stProductName,
  stVersionNumber, stStateName, stAttrCondition, stProjects,
  stLinkType: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.FindObjects(IdCheckOut, stTypeName, stProductName, stVersionNumber, stStateName,
        stAttrCondition, stProjects, stLinkType, Result)
    );
end;

function TAppMain.FindObjectsInContext2(IdCheckOut: Integer; stContext,
  stSearchXML: WideString; inParams: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.FindObjectsInContext2(IdCheckOut, stContext, stSearchXML, inParams, Result)
    );
end;

function TAppMain.GetTree(IdCheckOut: Integer; stTypeName, stProductName,
  stVersionNumber: WideString; inIdVersion: Integer; stLinkType: WideString;
  boWithAttrs: WordBool): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetTree(IdCheckOut, stTypeName, stProductName, stVersionNumber,
        inIdVersion, stLinkType, boWithAttrs, Result)
    );
end;

function TAppMain.GetTree2(IdCheckOut, inIdVersion: Integer; stLinkType: WideString;
  inParams: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetTree2(IdCheckOut, inIdVersion, stLinkType, inParams, Result)
    );
end;

function TAppMain.GetTypeList: IPluginDataSet;
begin
  ApiCheck(FAppMain.GetTypeList(Result));
end;

function TAppMain.GetPrivilegesForObjects(IdCheckOut: Integer; stIds: WideString;
  Mode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetPrivilegesForObjects(IdCheckOut, stIds, Mode, Result)
    );
end;

function TAppMain.GetProjectList2(IdCheckOut,
  inParams: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetProjectList2(IdCheckOut, inParams, Result)
    );
end;

function TAppMain.GetProjectListEx(IdCheckOut: Integer;
  boWithAttrs: WordBool): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetProjectListEx(IdCheckOut, boWithAttrs, Result)
    );
end;

function TAppMain.GetUserGroupList: IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetUserGroupList(Result)
    );
end;

function TAppMain.GetUserList: IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetUserList(Result)
    );
end;

function TAppMain.GetVersionList(IdCheckOut: Integer; stTypeName,
  stProductName: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetVersionList(IdCheckOut, stTypeName, stProductName, Result)
    );
end;

function TAppMain.GetVersionList2(IdCheckOut, inIdType: Integer;
  stProductName: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetVersionList2(IdCheckOut, inIdType, stProductName, Result)
    );
end;

function TAppMain.GetPropObjects2(IdCheckOut: Integer; stObjects: WideString;
  inParams: Integer): IPluginDataSet;
begin
  ApiCheck(FAppMain.GetPropObjects2(IdCheckOut, stObjects,
                    inParams, Result));
end;

function TAppMain.GetAllLinkedObjects(IdCheckOut: Integer; Ids: WideString;
  inParams: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetAllLinkedObjects(IdCheckOut, Ids, inParams, Result)
    );
end;

function TAppMain.GetAnnotations(AIdCheckout,
  AVersionId: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetAnnotations(AIdCheckout, AVersionId, Result)
    );
end;

function TAppMain.GetAttributeList: IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetAttributeList(Result)
    );
end;

function TAppMain.GetAttributesValues2(stIds,
  stAttrIds: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetAttributesValues2(stIds, stAttrIds, Result)
    );
end;

function TAppMain.GetAttrImageValueById(IdCheckOut, inIdVersion: Integer;
  stAttrName: WideString; inIdLink: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetAttrImageValueById(IdCheckOut, inIdVersion, stAttrName, inIdLink, Result)
    );
end;

function TAppMain.GetDBList: WideString;
begin
  FAppMain.GetDBList(Result);
end;

function TAppMain.GetInfoAboutAttribute(stAttrName: WideString;
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutAttribute(stAttrName, inMode, Result)
    );
end;

function TAppMain.GetInfoAboutCard(stCardName: WideString;
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutCard(stCardName, inMode, Result)
    );
end;

function TAppMain.GetInfoAboutCurrentBase(inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutCurrentBase(inMode, Result)
    );
end;

function TAppMain.GetInfoAboutCurrentUser: IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutCurrentUser(Result)
    );
end;

function TAppMain.GetPropObjects3(IdCheckOut: Integer; Ids,
  stAttrIds: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetPropObjects3(IdCheckOut, Ids, stAttrIds, Result)
    );
end;

function TAppMain.GetReport(IdCheckOut: Integer; ReportName,
  Params: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetReport(IdCheckOut, ReportName, Params, Result)
    );
end;

function TAppMain.GetSignatures(AIdCheckout,
  AVersionId: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetSignatures(AIdCheckout, AVersionId, Result)
    );
end;

function TAppMain.GetStateList: IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetStateList(Result)
    );
end;

function TAppMain.GetInfoAboutLink(IdCheckOut, inIdLink,
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(FAppMain.GetInfoAboutLink(IdCheckOut, inIdLink,
                    inMode, Result));
end;

function TAppMain.GetInfoAboutType(stTypeName: WideString;
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutType(stTypeName, inMode, Result)
    );
end;

function TAppMain.GetInfoAboutUserSet(stUserSet, stDBName: WideString;
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutUserSet(stUserSet, stDBName, inMode, Result)
    );
end;

function TAppMain.GetInfoAboutVersion(IdCheckOut: Integer; stTypeName,
  stProductName, stVersionNumber: WideString; inIdVersion,
  inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutVersion(IdCheckOut, stTypeName, stProductName,
      stVersionNumber, inIdVersion, inMode, Result)
    );
end;

function TAppMain.GetInfoAboutVersionsFiles(IdCheckOut: Integer;
  stIds: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutVersionsFiles(IdCheckOut, stIds, Result)
    );
end;

function TAppMain.GetInfoAboutVersionsPrivileges(stIds: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetInfoAboutVersionsPrivileges(stIds, Result)
    );
end;

function TAppMain.GetLinkAttrForTypes(stParentType, stChildType,
  stLinkType: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLinkAttrForTypes(stParentType, stChildType,
      stLinkType, Result)
    );
end;

function TAppMain.GetLinkAttributes(IdCheckOut,
  inIdLink: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLinkAttributes(IdCheckOut, inIdLink, Result)
    );
end;

function TAppMain.GetLinkedFast(IdCheckOut, inIdVersion: Integer;
  stLinkType: WideString; boInverse: WordBool): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLinkedFast(IdCheckOut, inIdVersion, stLinkType, boInverse, Result)
    );
end;

function TAppMain.GetLinkedFast2(IdCheckOut, inIdVersion: Integer;
  stLinkType: WideString; boInverse: WordBool): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLinkedFast2(IdCheckOut, inIdVersion, stLinkType, boInverse, Result)
    );
end;

function TAppMain.GetLinkedObjects2(IdCheckOut, inIdVersion: Integer;
  stLinkType: WideString; boInverse, boFullLink,
  boGroupByProduct: WordBool): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLinkedObjects2( IdCheckOut, inIdVersion, stLinkType, boInverse, boFullLink,
        boGroupByProduct, Result)
    );
end;

function TAppMain.GetLinkedObjectsAndFiles(IdCheckOut: Integer; stTypeName,
  stProductName, stVersionNumber, stLinkType: WideString; boFullLink,
  boViewOnlyDocuments: WordBool): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLinkedObjectsAndFiles(IdCheckOut, stTypeName, stProductName, stVersionNumber, stLinkType,
        boFullLink, boViewOnlyDocuments, Result)
    );
end;

function TAppMain.GetLinkedObjectsEx(IdCheckOut: Integer; stTypeName,
  stProductName, stVersionNumber: WideString; inIdVersion: Integer;
  stLinkType: WideString; boInverse, boFullLink,
  boGroupByProduct: WordBool): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLinkedObjectsEx(IdCheckOut, stTypeName, stProductName, stVersionNumber, inIdVersion,
        stLinkType, boInverse, boFullLink, boGroupByProduct, Result)
    );
end;

function TAppMain.GetLinks(IdCheckOut, inIdVersion1,
  inIdVersion2: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLinks(IdCheckOut, inIdVersion1, inIdVersion2, Result)
    );
end;

function TAppMain.GetLObjs(IdCheckOut, inIdVersion: Integer;
  boInverse: WordBool): IPluginDataSet;
begin
  ApiCheck(FAppMain.GetLObjs(IdCheckOut, inIdVersion,
            boInverse, Result));
end;

function TAppMain.GetLockedObjects(IdCheckOut,
  inParams: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.GetLockedObjects(IdCheckOut, inParams, Result)
    );
end;

function TAppMain.InsertObject(IdCheckOut: Integer; stParentType,
  stParentProduct, stParentVersion, stChildType, stChildProduct, stChildVersion,
  stLinkType, stState: WideString; boKeyInsert: WordBool): Integer;
begin
  ApiCheck(FAppMain.InsertObject(IdCheckOut, stParentType,
            stParentProduct, stParentVersion, stChildType, stChildProduct, stChildVersion,
            stLinkType, stState, boKeyInsert, Result));
end;

procedure TAppMain.IsProject(stTypeName, stProductName: WideString;
  boIsProject: WordBool);
begin
  ApiCheck(
    FAppMain.IsProject(stTypeName, stProductName, boIsProject)
      );
end;

procedure TAppMain.KillVersionById(IdCheckOut, inIdVersion: Integer);
begin
  ApiCheck(
    FAppMain.KillVersionById(IdCheckOut, inIdVersion)
    );
end;

function TAppMain.KillVersions(IdCheckOut: Integer; stIds: WideString;
  inParams: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.KillVersions(IdCheckOut, stIds, inParams, Result)
    );
end;

procedure TAppMain.Loodsman_ExtractFile(IdCheckout: Integer; TypeObj, Product,
  Version: WideString; IdVersion: Integer; stFileName, stLocalName: WideString; inSize,
  inCRC: Integer; dtCreate, dtModify: TDateTime; TargetFile: WideString);
begin
  ApiCheck(
    FAppMain.Loodsman_ExtractFile(IdCheckout, TypeObj, Product,
        Version, IdVersion, stFileName, stLocalName, inSize,
        inCRC, dtCreate, dtModify, TargetFile)
    );
end;

function TAppMain.NewObject(IdCheckOut: Integer; stTypeName, stStateName,
  stProductName: WideString; inProject: Integer): Integer;
begin
  ApiCheck(
    FAppMain.NewObject(IdCheckOut, stTypeName, stStateName, stProductName, inProject, Result)
    );
end;

procedure TAppMain.NewUserSet(stUserSet, stDBName, stParentUserSet: WideString);
begin
  ApiCheck(
    FAppMain.NewUserSet(stUserSet, stDBName, stParentUserSet)
    );
end;

function TAppMain.NewVersionEx(IdCheckOut: Integer; stTypeName, stProductName,
  stVersionNumber, stStateName, vaFileList: WideString; inGroup,
  inParams: Integer): Integer;
begin
  ApiCheck(
    FAppMain.NewVersionEx(IdCheckOut, stTypeName, stProductName, stVersionNumber, stStateName,
      vaFileList, inGroup, inParams, Result)
    );
end;

function TAppMain.OpenUserSet2(stUserSet: WideString;
  inParams: Integer): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.OpenUserSet2(stUserSet, inParams, Result)
    );
end;

procedure TAppMain.PrintFileToXPS(FileName, XPSFileName: WideString);
begin
  ApiCheck(
    FAppMain.PrintFileToXPS(FileName, XPSFileName)
    );
end;

procedure TAppMain.SetUserSetPattern(userSetName, dbName, xmlPattern: WideString);
begin
  ApiCheck(
    FAppMain.SetUserSetPattern(userSetName, dbName, xmlPattern)
    );
end;

procedure TAppMain.SignAs(const XPSFileName, Login: WideString);
begin
  ApiCheck(
    FAppMain.SignAs(XPSFileName, Login)
        );
end;

procedure TAppMain.UpAttrValue(IdCheckOut: Integer; stTypeName, stProductName,
  stVersionNumber, stAttrName, vaAttrValue, stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpAttrValue(IdCheckOut, stTypeName, stProductName,
        stVersionNumber, stAttrName, vaAttrValue, stIdUnit, boDel)
        );
end;

procedure TAppMain.UpAttrValue(IdCheckOut: Integer; stTypeName, stProductName,
  stVersionNumber, stAttrName: WideString; vaAttrValue: Integer; stIdUnit: WideString;
  boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpAttrValue(IdCheckOut, stTypeName, stProductName,
        stVersionNumber, stAttrName, vaAttrValue, stIdUnit, boDel)
        );
end;

procedure TAppMain.UpAttrValue(IdCheckOut: Integer; stTypeName, stProductName,
  stVersionNumber, stAttrName: WideString; vaAttrValue: Double; stIdUnit: WideString;
  boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpAttrValue(IdCheckOut, stTypeName, stProductName,
        stVersionNumber, stAttrName, vaAttrValue, stIdUnit, boDel)
        );
end;

procedure TAppMain.UpAttrValue(IdCheckOut: Integer; stTypeName, stProductName,
  stVersionNumber, stAttrName: WideString; vaAttrValue: TDateTime; stIdUnit: WideString;
  boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpAttrValue(IdCheckOut, stTypeName, stProductName,
        stVersionNumber, stAttrName, vaAttrValue, stIdUnit, boDel)
        );
end;

procedure TAppMain.UpAttrValueById(IdCheckOut, inIdVersion: Integer; stAttrName,
  vaAttrValue, stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpAttrValueById(IdCheckOut, inIdVersion,
      stAttrName, vaAttrValue, stIdUnit, boDel)
    );
end;

procedure TAppMain.UpAttrValueById(IdCheckOut, inIdVersion: Integer;
  stAttrName: WideString; vaAttrValue: Integer; stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpAttrValueById(IdCheckOut, inIdVersion,
      stAttrName, vaAttrValue, stIdUnit, boDel)
    );
end;

procedure TAppMain.UpAttrValueById(IdCheckOut, inIdVersion: Integer;
  stAttrName: WideString; vaAttrValue: Double; stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpAttrValueById(IdCheckOut, inIdVersion,
      stAttrName, vaAttrValue, stIdUnit, boDel)
    );
end;

procedure TAppMain.UpAttrValueById(IdCheckOut, inIdVersion: Integer;
  stAttrName: WideString; vaAttrValue: TDateTime; stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpAttrValueById(IdCheckOut, inIdVersion,
      stAttrName, vaAttrValue, stIdUnit, boDel)
    );
end;

procedure TAppMain.UpdateStateOnObjectById(IdCheckOut, inIdVersion: Integer;
  stStateName: WideString);
begin
  ApiCheck(
    FAppMain.UpdateStateOnObjectById(IdCheckOut, inIdVersion, stStateName)
    );
end;

procedure TAppMain.UpFindPattern(stFindPattern, stUserSet, stDBName,
  stTypePattern, stProductPattern, stVersionPattern, stStatePattern,
  stAttrCondition, stProjectPattern: WideString; nIdPattern: Integer; boActive,
  boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpFindPattern(stFindPattern, stUserSet, stDBName,
        stTypePattern, stProductPattern, stVersionPattern, stStatePattern,
        stAttrCondition, stProjectPattern, nIdPattern, boActive,
        boDel)
    );
end;

procedure TAppMain.UpGrantOnVersion(IdCheckOut: Integer; stTypeName,
  stProductName, stVersionNumber, stOrgUnitName: WideString; inPriveleges: Integer;
  dtFinishTime: TDateTime; boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpGrantOnVersion(IdCheckOut, stTypeName,
        stProductName, stVersionNumber, stOrgUnitName, inPriveleges,
        dtFinishTime, boDel)
    );
end;

function TAppMain.UpLink(IdCheckOut: Integer; stParentType, stParentProduct,
  stParentVersion, stChildType, stChildProduct, stChildVersion: WideString;
  inIdLink: Integer; stIdUnit: WideString; boDel: WordBool;
  stLinkType: WideString): Integer;
begin
  ApiCheck(
    FAppMain.UpLink(IdCheckOut, stParentType, stParentProduct,
    stParentVersion, stChildType, stChildProduct, stChildVersion,
    inIdLink, stIdUnit, boDel, stLinkType, Result)
    );
end;

procedure TAppMain.UpLinkAttrValue(IdCheckOut, inIdLink: Integer; stAttrName,
  vaAttrValue, stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
      FAppMain.UpLinkAttrValue(IdCheckOut, inIdLink, stAttrName,
            vaAttrValue, stIdUnit, boDel)
      );
end;

procedure TAppMain.UpLinkAttrValue(IdCheckOut, inIdLink: Integer;
  stAttrName: WideString; vaAttrValue: Integer; stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
      FAppMain.UpLinkAttrValue(IdCheckOut, inIdLink,
                  stAttrName, vaAttrValue, stIdUnit, boDel)
      );
end;

procedure TAppMain.UpLinkAttrValue(IdCheckOut, inIdLink: Integer;
  stAttrName: WideString; vaAttrValue: TDateTime; stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
      FAppMain.UpLinkAttrValue(IdCheckOut, inIdLink,
        stAttrName, vaAttrValue, stIdUnit, boDel)
      );
end;

procedure TAppMain.UpLinkAttrValue(IdCheckOut, inIdLink: Integer;
  stAttrName: WideString; vaAttrValue: Double; stIdUnit: WideString; boDel: WordBool);
begin
  ApiCheck(
      FAppMain.UpLinkAttrValue(IdCheckOut, inIdLink,
        stAttrName, vaAttrValue, stIdUnit, boDel)
      );
end;

function TAppMain.UpObject(IdCheckOut: Integer; stTypeName, stProductName,
  stNewProductName: WideString): Integer;
begin
  ApiCheck(
    FAppMain.UpObject(IdCheckOut, stTypeName, stProductName, stNewProductName, Result)
    );
end;

procedure TAppMain.UpUserSet(stUserSet, stDBName, stNewUserSet,
  stGroupName: WideString; inGroupOperation: Integer; boDel: WordBool);
begin
  ApiCheck(
    FAppMain.UpUserSet(stUserSet, stDBName, stNewUserSet,
        stGroupName, inGroupOperation, boDel)
    );
end;

function TAppMain.CheckOut(stTypeName, stProductName,
  stVersionNumber: WideString; inMode: Integer): Integer;
begin
  ApiCheck(FAppMain.CheckOut(stTypeName, stProductName,
      stVersionNumber, inMode, Result));
end;

function TAppMain.CheckUniqueName(stTypeName,
  stProductName: WideString): IPluginDataSet;
begin
  ApiCheck(
    FAppMain.CheckUniqueName(stTypeName, stProductName, Result)
    );
end;

procedure TAppMain.ClearTempUserDir;
begin
  ApiCheck(
    FAppMain.ClearTempUserDir()
    );
end;

procedure TAppMain.Client_CreateFile(TargetFile: WideString; IdCheckout,
  IdVersion: Integer; FileName: WideString);
begin
  ApiCheck(
    FAppMain.Client_CreateFile(TargetFile, IdCheckout, IdVersion, FileName)
    );
end;

procedure TAppMain.Client_DeleteFile(IdCheckout, IdVersion: Integer;
  FileName: WideString);
begin
  ApiCheck(
     FAppMain.Client_DeleteFile(IdCheckout, IdVersion, FileName)
    );
end;

function TAppMain.Client_GetFile(IdCheckout: Integer; stTypeObj, stProduct,
  stVersion, stFileName, stLocalName: WideString; inSize, inCRC: Integer; dtCreate,
  dtModify: TDateTime; RevertLocalChanges: WordBool): WideString;
begin
  ApiCheck(
    FAppMain.Client_GetFile(IdCheckout, stTypeObj, stProduct,
    stVersion, stFileName, stLocalName, inSize, inCRC, dtCreate,
    dtModify, RevertLocalChanges, Result)
    );
end;

function TAppMain.Client_GetFileById(IdCheckout, IdVersion: Integer; stFileName,
  stLocalName: WideString; inSize, inCRC: Integer; dtCreate, dtModify: TDateTime;
  RevertLocalChanges: WordBool): WideString;
begin
  ApiCheck(
    FAppMain.Client_GetFileById(IdCheckout, IdVersion, stFileName,
      stLocalName, inSize, inCRC, dtCreate, dtModify,
      RevertLocalChanges, Result)
    );
end;

function TAppMain.Client_GetFilePath(IdVersion: Integer;
  stFileName: WideString): WideString;
begin
  ApiCheck(
    FAppMain.Client_GetFilePath(IdVersion, stFileName, Result)
    );
end;

procedure TAppMain.Client_UpdateFile(TargetFile: WideString; IdCheckout,
  IdVersion: Integer; FileName: WideString);
begin
  ApiCheck(
    FAppMain.Client_UpdateFile(TargetFile, IdCheckout, IdVersion, FileName)
    );
end;

procedure TAppMain.CheckIn(IdCheckOut: Integer);
begin
  ApiCheck(FAppMain.CheckIn(IdCheckOut));
end;

procedure TAppMain.CancelCheckOut(IdCheckOut: Integer);
begin
  ApiCheck(FAppMain.CancelCheckOut(IdCheckOut));
end;


{ TAppWorkFlow }

constructor TAppWorkFlow.Create(AAppWorkFlow: IPluginAppWorkFlow);
begin
  FAppWorkFlow := AAppWorkFlow;
end;

procedure TAppWorkFlow.ApiCheck(AResult: WordBool);
begin
  if not AResult then
    raise EApiException.Create(FAppWorkFlow.GetLastError);
end;

procedure TAppWorkFlow.AddBPDocument(inIdRoute, inIdDoc, inIdPar: Integer);
begin
  ApiCheck(
      FAppWorkFlow.AddBPDocument(inIdRoute, inIdDoc, inIdPar)
      );
end;

procedure TAppWorkFlow.AddRouteItemsRetSeq(inIdFirst, inIdSecond: Integer);
begin
  ApiCheck(
      FAppWorkFlow.AddRouteItemsRetSeq(inIdFirst, inIdSecond)
      );
end;

procedure TAppWorkFlow.AddRouteItemsSeq(inIdFirst, inIdSecond, inType: Integer);
begin
  ApiCheck(
      FAppWorkFlow.AddRouteItemsSeq(inIdFirst, inIdSecond, inType)
      );
end;

function TAppWorkFlow.CreateNewProcess(stDBName: WideString;
  inID_PDM_OBJECT: Integer; stRouteName: WideString): Integer;
begin
  ApiCheck(
      FAppWorkFlow.CreateNewProcess(stDBName,
            inID_PDM_OBJECT, stRouteName, Result)
      );
end;

function TAppWorkFlow.CreateTask(Subj, Text: WideString; IdVersion, IdRole,
  IdUser: Integer; DatePlan: TDateTime; CriticalTerm: Integer;
  CheckResult: WordBool; Kind: TBProcessKindExp): Integer;
begin
  ApiCheck(
      FAppWorkFlow.CreateTask(Subj, Text, IdVersion, IdRole,
          IdUser, DatePlan, CriticalTerm,
          CheckResult, Kind, Result)
      );
end;

function TAppWorkFlow.CreateTaskByDialog: Integer;
begin
  ApiCheck(
      FAppWorkFlow.CreateTaskByDialog(Result)
      );
end;

procedure TAppWorkFlow.DelBPDocument(inIdRoute, inIdDoc: Integer);
begin
  ApiCheck(
      FAppWorkFlow.DelBPDocument(inIdRoute, inIdDoc)
      );
end;

procedure TAppWorkFlow.DeleteProcess(inID_Route: Integer);
begin
  ApiCheck(
      FAppWorkFlow.DeleteProcess(inID_Route)
      );
end;

procedure TAppWorkFlow.DelLetter(inIDLetter: Integer);
begin
  ApiCheck(
      FAppWorkFlow.DelLetter(inIDLetter)
      );
end;

function TAppWorkFlow.GetAttachDocumentListIntoProcess(
  inIdRoute: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetAttachDocumentListIntoProcess(inIdRoute, Result)
      );
end;

function TAppWorkFlow.GetCorrespondenceList(
  inID_Route: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetCorrespondenceList(inID_Route, Result)
      );
end;

function TAppWorkFlow.GetFirstItems(inIdRoute: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetFirstItems(inIdRoute, Result)
      );
end;

function TAppWorkFlow.GetIncomingCorrespondence(
  inRetMode: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetIncomingCorrespondence(inRetMode, Result)
      );
end;

function TAppWorkFlow.GetInfoAboutCurrentUser: IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetInfoAboutCurrentUser(Result)
      );
end;

function TAppWorkFlow.GetInfoAboutRoute(inID_Route: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetInfoAboutRoute(inID_Route, Result)
      );
end;

function TAppWorkFlow.GetInfoAboutStage(inIdStage: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetInfoAboutStage(inIdStage, Result)
      );
end;

function TAppWorkFlow.GetInitialProcessList: IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetInitialProcessList(Result)
      );
end;

function TAppWorkFlow.GetJobsListIntoProcess(
  inID_Route: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetJobsListIntoProcess(inID_Route, Result)
      );
end;

function TAppWorkFlow.GetNextItems(inIdItem: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetNextItems(inIdItem, Result)
      );
end;

function TAppWorkFlow.GetOutcomingCorrespondence(
  inRetMode: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetOutcomingCorrespondence(inRetMode, Result)
      );
end;

function TAppWorkFlow.GetProcessListByObject(stDBName: WideString;
  inPdm_Id: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetProcessListByObject(stDBName, inPdm_Id, Result)
      );
end;

function TAppWorkFlow.GetProcessListByObjectEx(inPdm_Id: Integer;
  stDBName: WideString): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetProcessListByObjectEx(inPdm_Id, stDBName, Result)
      );
end;

function TAppWorkFlow.GetRoleListForUser(inID_User: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetRoleListForUser(inID_User, Result)
      );
end;

function TAppWorkFlow.GetRoleListIntoProcess(
  inID_Route: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetRoleListIntoProcess(inID_Route, Result)
      );
end;

function TAppWorkFlow.GetRoleTree(inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetRoleTree(inMode, Result)
      );
end;

function TAppWorkFlow.GetTaskListForObject(inIdVersion, inProcessStateMask,
  inStageStateMode: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetTaskListForObject(inIdVersion, inProcessStateMask,
          inStageStateMode, Result)
      );
end;

function TAppWorkFlow.GetTaskListForUser(inIdUser, inUserRoleMask,
  inProcessStateMask, inStageStateMode: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetTaskListForUser(inIdUser, inUserRoleMask,
        inProcessStateMask, inStageStateMode, Result)
      );
end;

function TAppWorkFlow.GetTypicalProcessList(inMode: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetTypicalProcessList(inMode, Result)
      );
end;

function TAppWorkFlow.GetUserList: IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.GetUserList(Result)
      );
end;

procedure TAppWorkFlow.GetVariable(inIdRoute: Integer; stName: WideString;
  out res: WideString);
begin
  ApiCheck(
      FAppWorkFlow.GetVariable(inIdRoute, stName, res)
      );
end;

procedure TAppWorkFlow.GetVariable(inIdRoute: Integer; stName: WideString;
  out res: Integer);
begin
  ApiCheck(
      FAppWorkFlow.GetVariable(inIdRoute, stName, res)
      );
end;

procedure TAppWorkFlow.GetVariable(inIdRoute: Integer; stName: WideString;
  out res: Double);
begin
  ApiCheck(
      FAppWorkFlow.GetVariable(inIdRoute, stName, res)
      );
end;

procedure TAppWorkFlow.GetVariable(inIdRoute: Integer; stName: WideString;
  out res: TDateTime);
begin
  ApiCheck(
      FAppWorkFlow.GetVariable(inIdRoute, stName, res)
      );
end;

procedure TAppWorkFlow.InsUpdStageIntoBP(stTask: WideString;
  inCriticalTerm: Integer; dtDatePlan: TDateTime; inCanControl, inActive,
  inIdRole, inIdUser, inIdBProc, inCanTransform: Integer; out inIdStage,
  inTimeConstraint, inFromBPStart: Integer);
begin
  ApiCheck(
      FAppWorkFlow.InsUpdStageIntoBP(stTask,
          inCriticalTerm, dtDatePlan, inCanControl, inActive,
          inIdRole, inIdUser, inIdBProc, inCanTransform, inIdStage,
          inTimeConstraint, inFromBPStart)
      );
end;

procedure TAppWorkFlow.InsUpdUser(stUserName, stDescr, stEMail: WideString;
  inIsAdmin: Integer; out vaIdUser: Integer);
begin
  ApiCheck(
      FAppWorkFlow.InsUpdUser(stUserName, stDescr, stEMail,
          inIsAdmin, vaIdUser)
      );
end;

procedure TAppWorkFlow.RenameProcess(inID_Route: Integer; stNewName: WideString);
begin
  ApiCheck(
      FAppWorkFlow.RenameProcess(inID_Route, stNewName)
      );
end;

function TAppWorkFlow.rep_ExecutedJobs(inID_Parent: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppWorkFlow.rep_ExecutedJobs(inID_Parent, Result)
      );
end;

function TAppWorkFlow.SendLetterByBPToOneUser(inIdRoute, inIdAURole,
  inIdRecRole, inIdRecUser: Integer; stSubj, stBody: WideString): Integer;
begin
  ApiCheck(
      FAppWorkFlow.SendLetterByBPToOneUser(inIdRoute, inIdAURole,
          inIdRecRole, inIdRecUser, stSubj, stBody, Result)
      );
end;

function TAppWorkFlow.SendLetterByBPToUsers(inIdRoute, inIdAURole: Integer;
  vaRecipientList: IPluginDataSet; stSubj, stBody: WideString): Integer;
begin
  ApiCheck(
      FAppWorkFlow.SendLetterByBPToUsers(inIdRoute, inIdAURole,
          vaRecipientList, stSubj, stBody, Result)
      );
end;

procedure TAppWorkFlow.SetBP_PDMObject(inIdRoute, inIdPDMObject: Integer);
begin
  ApiCheck(
      FAppWorkFlow.SetBP_PDMObject(inIdRoute, inIdPDMObject)
      );
end;

procedure TAppWorkFlow.SetItemState(inIdItem, inState, inAddSignature: Integer);
begin
  ApiCheck(
      FAppWorkFlow.SetItemState(inIdItem, inState, inAddSignature)
      );
end;

procedure TAppWorkFlow.SetItemStateInternal(inIdItem, inState: Integer);
begin
  ApiCheck(
      FAppWorkFlow.SetItemStateInternal(inIdItem, inState)
      );
end;

procedure TAppWorkFlow.SetStageExecutor(inIdStage, inIdUser, inIdRole: Integer);
begin
  ApiCheck(
      FAppWorkFlow.SetStageExecutor(inIdStage, inIdUser, inIdRole)
      );
end;

procedure TAppWorkFlow.SetStageTask(inIdStage: Integer; stTask: WideString);
begin
  ApiCheck(
      FAppWorkFlow.SetStageTask(inIdStage, stTask)
      );
end;

procedure TAppWorkFlow.SetStageTerm(inIdStage, inTimeLimit: Integer;
  vaPlanEnd: TDateTime; inTimeConstraint, inFromBPStart: Integer);
begin
  ApiCheck(
      FAppWorkFlow.SetStageTerm(inIdStage, inTimeLimit,
          vaPlanEnd, inTimeConstraint, inFromBPStart)
      );
end;

procedure TAppWorkFlow.SetStateOfProcess(inID_Route, inStateBP: Integer);
begin
  ApiCheck(
      FAppWorkFlow.SetStateOfProcess(inID_Route, inStateBP)
      );
end;

procedure TAppWorkFlow.SetTaskState(inID_Route, inID_Operation,
  inStateTask: Integer; stMessageBody: WideString);
begin
  ApiCheck(
      FAppWorkFlow.SetTaskState(inID_Route, inID_Operation, inStateTask, stMessageBody)
      );
end;

procedure TAppWorkFlow.SetVariable(inIdRoute: Integer; stName: WideString;
  Value: Double; ReadOnlyForChild: WordBool);
begin
  ApiCheck(
      FAppWorkFlow.SetVariable(inIdRoute, stName, Value, ReadOnlyForChild)
      );
end;

procedure TAppWorkFlow.SetVariable(inIdRoute: Integer; stName: WideString;
  Value: TDateTime; ReadOnlyForChild: WordBool);
begin
  ApiCheck(
      FAppWorkFlow.SetVariable(inIdRoute, stName, Value, ReadOnlyForChild)
      );
end;

procedure TAppWorkFlow.SetVariable(inIdRoute: Integer; stName: WideString;
  Value: Integer; ReadOnlyForChild: WordBool);
begin
  ApiCheck(
      FAppWorkFlow.SetVariable(inIdRoute, stName, Value, ReadOnlyForChild)
      );
end;

procedure TAppWorkFlow.SetVariable(inIdRoute: Integer; stName, Value: WideString;
  ReadOnlyForChild: WordBool);
begin
  ApiCheck(
      FAppWorkFlow.SetVariable(inIdRoute, stName, Value, ReadOnlyForChild)
      );
end;

procedure TAppWorkFlow.StartNextStage(inID_Route, inIdItem, inDirect: Integer);
begin
  ApiCheck(
      FAppWorkFlow.StartNextStage(inID_Route, inIdItem, inDirect)
      );
end;

procedure TAppWorkFlow.StartTask(IdRoute, IdRole, IdUser: Integer; Subj: WideString;
  DatePlan: TDateTime);
begin
  ApiCheck(
      FAppWorkFlow.StartTask(IdRoute, IdRole, IdUser, Subj, DatePlan)
      );
end;

{ TAppConfigurator }

constructor TAppConfigurator.Create(AAppConfigurator: IPluginAppConfigurator);
begin
  FAppConfigurator := AAppConfigurator;
end;

procedure TAppConfigurator.ApiCheck(AResult: WordBool);
begin
  if not AResult then
    raise EApiException.Create(FAppConfigurator.GetLastError);
end;

function TAppConfigurator.GetSortCase(stProfile, stType, stLink: WideString;
  inIndex: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppConfigurator.GetSortCase(stProfile, stType, stLink, inIndex, Result)
      );
end;

function TAppConfigurator.GetSortTable(inIdCase: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppConfigurator.GetSortTable(inIdCase, Result)
      );
end;

function TAppConfigurator.GetSortTree(inIdCase: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppConfigurator.GetSortTree(inIdCase, Result)
      );
end;

function TAppConfigurator.GetSortTypes(inIdCase: Integer): IPluginDataSet;
begin
  ApiCheck(
      FAppConfigurator.GetSortTypes(inIdCase, Result)
      );
end;


{ TAppProvider }

constructor TAppProvider.Create(AAppProvider: IPluginAppProvider);
begin
  FAppProvider := AAppProvider;
  FAppMain := TAppMain.Create(AAppProvider.GetAppMain);
  FAppWorkFlow := TAppWorkFlow.Create(AAppProvider.GetAppWorkFlow);
  FAppCache := TAppCache.Create(AAppProvider.GetAppCache);
  FAppConfigurator := TAppConfigurator.Create(AAppProvider.GetAppConfigurator);
end;

destructor TAppProvider.Destroy;
begin
  FreeAndNil(FAppMain);
  FreeAndNil(FAppWorkFlow);
  FreeAndNil(FAppCache);
  FreeAndNil(FAppConfigurator);
  inherited;
end;

procedure TAppProvider.ApiCheck(AResult: WordBool);
begin
  if not AResult then
    raise EApiException.Create(FAppProvider.GetLastError);
end;

function TAppProvider.GetAppHandle: THandle;
begin
  Result := FAppProvider.GetAppHandle;
end;

function TAppProvider.GetAppTitle: WideString;
begin
  Result := FAppProvider.GetAppTitle;
end;

function TAppProvider.GetAttachFromDlg(
  out Versions: IPluginPDMVersions): TModalResultExp;
begin
  ApiCheck(
      FAppProvider.GetAttachFromDlg(Versions, Result)
      );
end;

function TAppProvider.GetSelectedObjects: IPluginPDMVersions;
begin
  Result := FAppProvider.GetSelectedObjects;
end;

function TAppProvider.GetSelectedTasks: IPluginTasks;
begin
  ApiCheck(
      FAppProvider.GetSelectedTasks(Result)
      );
end;

function TAppProvider.GetUsersFromDlg(ADateVisible, ASignRoleVisible,
  AMultiselect, ACan_I_Am: Boolean; ASelectedUsers: IPluginUsers;
  out Users: IPluginUsers): TModalResultExp;
begin
  ApiCheck(
      FAppProvider.GetUsersFromDlg(ADateVisible, ASignRoleVisible,
      AMultiselect, ACan_I_Am, ASelectedUsers, Users, Result)
      );
end;

procedure TAppProvider.RefreshTasks(ASelectedRouteId: Integer);
begin
  ApiCheck(
      FAppProvider.RefreshTasks(ASelectedRouteId)
      );
end;

procedure TAppProvider.RefreshVersion(IdVersion: Integer);
begin
  FAppProvider.RefreshVersion(IdVersion);
end;

procedure TAppProvider.SelectMail(IdLetter: Integer;
  Filter: TMailFilterExp);
begin
  ApiCheck(
      FAppProvider.SelectMail(IdLetter, Filter)
      );
end;

procedure TAppProvider.SelectObject(IdVersion: Integer);
begin
  ApiCheck(
      FAppProvider.SelectObject(IdVersion)
      );
end;

procedure TAppProvider.SelectTask(IdProcess: Integer;
  TasksFilter: TTasksFilterExp);
begin
  ApiCheck(
      FAppProvider.SelectTask(IdProcess, TasksFilter)
      );
end;

end.
