unit Plugin;

interface

uses
  Windows,
  SysUtils,

  PluginIntf,
  PluginIntfWrapper;


type
  TPlugin = class(TInterfacedObject, IPlugin, IPluginCoolCommands)
  private
    FLastError: WideString;
    FAppProvider: TAppProvider;
    function IsSelectedProject: Boolean;
  private
    procedure ShowException(const AMessage: string);
  private
    // IPlugin
    function GetVersion(): Integer; safecall;
    function GetLastError(): WideString; safecall;
    procedure Init(AppProvider: IPluginAppProvider); safecall;
    // IPluginCoolCommands
    procedure InitCoolCommands(ACCPanel: IPluginCoolCommandsPanel); safecall;
    procedure CallCoolCommand(AIndex: Integer); safecall;
  public
    constructor Create;
    destructor Destroy; override;
  end;


function LoadPlugin(): IPlugin; safecall;


implementation

uses
  Forms,
  Dialogs;


function LoadPlugin(): IPlugin;
begin
  Result := TPlugin.Create;
end;


{ TPlugin }

constructor TPlugin.Create;
begin
end;

destructor TPlugin.Destroy;
begin
  //   0  Application.Handle
  Application.Handle := 0;
  if Assigned(FAppProvider) then
    FreeAndNil(FAppProvider);
  inherited;
end;

procedure TPlugin.ShowException(const AMessage: string);
begin
  Application.MessageBox(
    PChar(AMessage),
    PChar(' '),
    MB_ICONERROR or MB_OK or MB_TOPMOST);
end;

function TPlugin.GetLastError: WideString;
begin
  Result := FLastError;
end;

function TPlugin.GetVersion: Integer;
begin
  Result := PLUGIN_VERSION;
end;

procedure TPlugin.Init(AppProvider: IPluginAppProvider);
begin
  try
    FAppProvider := TAppProvider.Create(AppProvider);
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.InitCoolCommands(ACCPanel: IPluginCoolCommandsPanel);
begin
  try
    //      
    ACCPanel.AddCommand2(cSpaceDocuments, cGroupObject, '   ', 'IMAGE_1', 0);

    //    ,    
    ACCPanel.AddCommand2(cSpaceDocuments, cGroupObject, ' 1', 'IMAGE_2', 1);

    //    ,     
    if IsSelectedProject then
    begin
      ACCPanel.AddCommand2(cSpaceDocuments, ' 1', ' 2', 'IMAGE_3', 2);
      //      
      ACCPanel.AddCommand2(cSpaceDocuments, ' 1', ' 2', '', 3);
    end;
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

function TPlugin.IsSelectedProject: Boolean;
//   True         ""
var
  i: Integer;
  SelectedObjects: IPluginPDMVersions;
begin
  Result := False;

  SelectedObjects := FAppProvider.GetSelectedObjects;
  if SelectedObjects.Count = 0 then
    Exit;

  Result := True;
  for i := 0 to SelectedObjects.Count - 1 do
    if not Assigned(SelectedObjects.Version(i)) or
       (SelectedObjects.Version(i).TypeObj <> '') then
      begin
        Result := False;
        Break;
      end;
end;

procedure TPlugin.CallCoolCommand(AIndex: Integer);
begin
  try
    //  AppHandle      
    // !     0  Application.Handle
    Application.Handle := FAppProvider.GetAppHandle;

    case AIndex of
      0: ShowMessage('  1');
      1: ShowMessage('  2');
      2: ShowMessage('  3');
      // ...   
    end;
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

end.
