﻿unit DlgTaskEdit;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Controls,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  ActnList,
  Dialogs,
  Forms,
  Buttons,
  ToolWin,

  PluginIntf,
  PluginIntfWrapper,
  FrameAttachments,
  PluginUsers,
  MiscApp;


type
  TfrmDlgTaskEdit = class(TForm)
    ActionList1: TActionList;
    AEditTask: TAction;
    AAttach: TAction;
    pnlTop: TPanel;
    Label1: TLabel;
    edtSabj: TEdit;
    btnRecipient: TButton;
    Splitter1: TSplitter;
    pnlAttachments: TPanel;
    pnlPlanning: TPanel;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton3: TToolButton;
    Edit1: TEdit;
    edtText: TMemo;
    procedure AEditTaskExecute(Sender: TObject);
    procedure AAttachExecute(Sender: TObject);
    procedure btnRecipientClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FAppProvider: TAppProvider;
    FTaskHalper: TTaskHalper;
    FFrameAttachments: TfrmFrameAttachments;
    FTask: IPluginTask;
    FUsers: IPluginUsers;
    FAttachments: IPluginPDMVersions;
    function GetAttachments(ARouteId: Integer): IPluginPDMVersions;
    procedure LoadUsers(AUser: IPluginUser);
    function WasChange(const ANewSubject, ANewText: string;
      ANewPlanDate: TDateTime; ANewIdRole, ANewIdExecutor: Integer;
      AAttachments: IPluginPDMVersions): Boolean;
    procedure DoEdit(const ANewSubject, ANewText: string;
      ANewPlanDate: TDateTime; ANewIdRole, ANewIdExecutor: Integer;
      AAttachments: IPluginPDMVersions);
    procedure ClearAttachments;
    procedure AddAttachments(AAttachments: IPluginPDMVersions);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent; AAppProvider: TAppProvider); reintroduce;
    destructor Destroy; override;
  end;

implementation

{$R *.dfm}

uses
  IniFiles,

  MiscGUI,
  PluginPDMVersionList;

type
  TBProcessState = (psNew, psActive, psStopped, psFinished);

  // Информация о процессе закодированная в названии
  TBProcessInfo = record
    Kind: TBProcessKindExp;
    Title: string;
  end;


constructor TfrmDlgTaskEdit.Create(AOwner: TComponent;
  AAppProvider: TAppProvider);
begin
  inherited Create(AOwner);

  FAppProvider := AAppProvider;
  FTaskHalper := TTaskHalper.Create(FAppProvider);

  FFrameAttachments := TfrmFrameAttachments.Create(Self, FAppProvider);
  PlaceFrameToControl(FFrameAttachments, pnlAttachments);

  Edit1.ReadOnly := True;
end;

destructor TfrmDlgTaskEdit.Destroy;
begin
  FreeAndNil(FTaskHalper);
  inherited;
end;

procedure TfrmDlgTaskEdit.FormShow(Sender: TObject);
begin
  FTask := FAppProvider.GetSelectedTasks.Task(0);

  Caption := Format('Редактирование задания: %s', [FTask.Subject]);
  edtSabj.Text := FTask.Subject;
  edtText.Text := FTask.Text;

  LoadUsers(FTask.Executor);

  if FTask.HasAttachments then
  begin
    FAttachments := GetAttachments(FTask.RouteId);
    FFrameAttachments.Load(FAttachments);
  end;
end;

function TfrmDlgTaskEdit.GetAttachments(ARouteId: Integer): IPluginPDMVersions;

  function GetValueList(Ads: IPluginDataSet; const AFieldName: string): string;
  // Возвращает список значений через запятую
  begin
    Result := '';
    Ads.First;
    while not Ads.Eof do
    begin
      Result := AddStringWithSeparator(Result, Ads.AsString(AFieldName), ',');
      Ads.Next;
    end;
  end;

var
  ds: IPluginDataSet;
  Ids: string;
begin
  ds := FAppProvider.AppWorkFlow.GetAttachDocumentListIntoProcess(ARouteId);
  Ids := GetValueList(ds, '_PDM_OBJECT_ID');

  if not IsNull(Ids) then
    Result := FAppProvider.AppCache.LoadPDMVersionsByIds(Ids)
  else
    Result := nil;
end;

procedure TfrmDlgTaskEdit.AAttachExecute(Sender: TObject);
begin
  FFrameAttachments.ShowDlgAttach;
end;

procedure TfrmDlgTaskEdit.btnRecipientClick(Sender: TObject);
var
  Users: IPluginUsers;
begin
  if FAppProvider.GetUsersFromDlg(True, False, False, False, FUsers, Users) = mreOk then
  begin
    LoadUsers(Users.User(0));
    FUsers := Users;
  end;
end;

procedure TfrmDlgTaskEdit.LoadUsers(AUser: IPluginUser);
var
  Info: string;
begin
  Info := ConvertToUserTitle(AUser.UserName, AUser.FullUserName, False);
  if not IsNull(AUser.PlanDate) then
    Info := Format('%s (%s)', [Info, DateToStr(AUser.PlanDate)]);

  Edit1.Text := Info;
end;

function TfrmDlgTaskEdit.WasChange(const ANewSubject, ANewText: string;
      ANewPlanDate: TDateTime; ANewIdRole, ANewIdExecutor: Integer;
      AAttachments: IPluginPDMVersions): Boolean;
begin
  Result := (ANewIdExecutor <> FTask.Executor.IdUser) or
            (ANewSubject <> FTask.Subject) or
            (ANewPlanDate <> FTask.Executor.PlanDate) or
            (ANewText <> FTask.Text) or
            not EqualsIds(AAttachments, FAttachments);
end;

procedure TfrmDlgTaskEdit.AEditTaskExecute(Sender: TObject);
begin
  if WasChange(edtSabj.Text,
              edtText.Text,
              FUsers.User(0).PlanDate,
              FUsers.User(0).IdRole,
              FUsers.User(0).IdUser,
              FFrameAttachments.GetAttachments) then
  begin
    DoEdit(edtSabj.Text,
           edtText.Text,
           FUsers.User(0).PlanDate,
           FUsers.User(0).IdRole,
           FUsers.User(0).IdUser,
           FFrameAttachments.GetAttachments);

    FAppProvider.RefreshTasks(FTask.RouteId);
  end;

  ModalResult := mrOk;
end;

procedure TfrmDlgTaskEdit.DoEdit(const ANewSubject, ANewText: string;
  ANewPlanDate: TDateTime; ANewIdRole, ANewIdExecutor: Integer;
  AAttachments: IPluginPDMVersions);

  function ProcessKindToInt(Kind: TBProcessKindExp): Integer;
  // Конвертирование типа процесса в число
  begin
    Result := 255;

    if IsPGS(FAppProvider.GetAppTitle) then
      case Kind of
        bpkTask:
          Result := 2;
        bpkTaskAgree:
          Result := 3;
      end
    else
    if IsORD(FAppProvider.GetAppTitle) then
      case Kind of
        bpkTask:
          Result := 4;
        bpkTaskAgree:
          Result := 5;
        bpkTaskFamiliarization:
          Result := 6;
      end;
  end;

  function BPInfoToText(const Info: TBProcessInfo): string;
  // Упаковка информации о бизнес-процессе в текст
  begin
    Result := Format('{%.2x%.8x}%s',
                     [ProcessKindToInt(Info.Kind), 0, Info.Title]);
  end;

  function GetLetterInfo(const ASubj: string; APlanDate: TDateTime): string;
  const
    SECT_BP_INFO = 'BPInfo';
    IDENT_SUBJ = 'Subj';
    IDENT_DATEPLAN = 'DatePlan';
  var
    ini: TMemIniFile;
    SL: TStrings;
  begin
    ini := TMemIniFile.Create('');
    SL := TStringList.Create;
    try
      ini.WriteString(SECT_BP_INFO, IDENT_SUBJ, ASubj);
      if APlanDate <> 0 then
        ini.WriteDateTime(SECT_BP_INFO, IDENT_DATEPLAN, APlanDate);
      ini.GetStrings(SL);
      Result := SL.Text;
    finally
      SL.Free;
      ini.Free;
    end;
  end;

const
  ch_Subject = 'Изменена тема задания';
  ch_Text = 'Изменен текст задания';
  ch_PlanDate = 'Изменен срок выполнения';
  ch_Executor = 'Изменен исполнитель';
  ch_Attachments = 'Изменено вложение';

  cNewTask = '{NewTask}';
var
  sl: TStrings;
  Info: TBProcessInfo;
begin
  sl := TStringList.Create;
  try
    // Сперва остановим процесс. Запланированное задание останавливать нет смысла
    if FTask.State <> tsPlanned then
      FAppProvider.AppWorkFlow.SetStateOfProcess(FTask.RouteId, Ord(psStopped));

    if ANewIdExecutor <> FTask.Executor.IdUser then
    begin
      FAppProvider.AppWorkFlow.SetStageExecutor(FTask.OperationId, ANewIdExecutor, ANewIdRole);

      if FTask.State = tsInWork then
        // возвращаем состояние в "Выдано"
        FAppProvider.AppWorkFlow.SetItemState(FTask.OperationId, 2 , 0);
      sl.Append(ch_Executor);
    end;

    if ANewSubject <> FTask.Subject then
    begin
      Info.Kind := FTask.ProcessKind;
      Info.Title := ANewSubject;
      FAppProvider.AppWorkFlow.RenameProcess(FTask.RouteId, BPInfoToText(Info));
      sl.Append(ch_Subject);
    end;

    if ANewPlanDate <> FTask.Executor.PlanDate then
    begin
      FAppProvider.AppWorkFlow.SetStageTerm(FTask.OperationId, 0, ANewPlanDate, 0, 0);
      sl.Append(ch_PlanDate);
    end;

    if ANewText <> FTask.Text then
    begin
      FAppProvider.AppWorkFlow.SetStageTask(FTask.OperationId, ANewText);
      sl.Append(ch_Text);
    end;

    if not EqualsIds(AAttachments, FAttachments) then
    begin
      ClearAttachments;
      AddAttachments(AAttachments);
      sl.Append(ch_Attachments);
    end;

    if FTask.State <> tsPlanned then
    begin
      FAppProvider.AppWorkFlow.SetStateOfProcess(FTask.RouteId, Ord(psActive));

      if sl.Text <> '' then
        // даже при смене исполнителя отправляем сообщение старому исполнителю
        FAppProvider.AppWorkFlow.SendLetterByBPToOneUser(
                                      FTask.RouteId, 0, FTask.Executor.IdRole,
                                      FTask.Executor.IdUser,
                                      ANewSubject,
                                      sl.Text);

      if ANewIdExecutor <> FTask.Executor.IdUser then
        // при смене исполнителя, сообщаем новому исполнителю что у него
        // новое задание. Отправить сообщение можно только после запуска БП
        FAppProvider.AppWorkFlow.SendLetterByBPToOneUser(
                                      FTask.RouteId, 0, ANewIdRole,
                                      ANewIdExecutor,
                                      cNewTask,
                                      GetLetterInfo(ANewSubject, ANewPlanDate));
    end;
  finally
    sl.Free
  end;
end;

procedure TfrmDlgTaskEdit.ClearAttachments;
var
  i: Integer;
begin
  if not Assigned(FAttachments) then
    Exit;

  // сперва удаляем основной объект
  FAppProvider.AppWorkFlow.SetBP_PDMObject(FTask.RouteId, 0);

  for i := 0 to FAttachments.Count - 1 do
    FAppProvider.AppWorkFlow.DelBPDocument(FTask.RouteId, FAttachments.Version(i).IdVersion);
end;

procedure TfrmDlgTaskEdit.AddAttachments(AAttachments: IPluginPDMVersions);
var
  i: Integer;
  FirstIdAttachment: Integer;
begin
  if not Assigned(AAttachments) then
    Exit;

  if AAttachments.Count = 0 then
    Exit;

  FirstIdAttachment := FTaskHalper.GetFirstIdAttachment(AAttachments);

  FAppProvider.AppWorkFlow.SetBP_PDMObject(FTask.RouteId, FirstIdAttachment);

  for i := 0 to AAttachments.Count - 1 do
    FAppProvider.AppWorkFlow.AddBPDocument(FTask.RouteId, AAttachments.Version(i).IdVersion, 0);
end;

end.
