unit FrameAttachments;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ComCtrls,
  ActnList,
  Menus,
  ImgList,

  snlListView,
  PluginIntf,
  PluginIntfWrapper,
  PluginPDMVersionList,
  ImagesCache;


type
  TfrmFrameAttachments = class(TFrame)
    lvAttachments: TListView;
    ActionList1: TActionList;
    AAttach: TAction;
    ADeleteAttachment: TAction;
    ASecurity: TAction;
    APropertiesAttachment: TAction;
    ilLargeTypes: TImageList;
    procedure lvAttachmentsAdvancedCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; Stage: TCustomDrawStage;
      var DefaultDraw: Boolean);
    procedure lvAttachmentsResize(Sender: TObject);
    procedure AAttachExecute(Sender: TObject);
  private
    FAppProvider: TAppProvider;
    FImagesCache: TImagesCache;
    FAttachments: TPluginPDMVersionList;
    FReadOnly: Boolean;
    FOnChange: TNotifyEvent;
    procedure DeleteSelectedAttachments;
    procedure DoChange;
    function GetCount: Integer;
    procedure AddItem(AVersion: IPluginPDMVersion);
    function GetAttachment(Index: Integer): IPluginPDMVersion;
  protected
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent; AAppProvider: TAppProvider); reintroduce;
    destructor Destroy; override;
    procedure Clear;
    procedure ShowDlgAttach;
    property ReadOnly: Boolean read FReadOnly write FReadOnly;
    property Count: Integer read GetCount;
    property Attachment[Index: Integer]: IPluginPDMVersion read GetAttachment; default;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    procedure Load(AVersions: IPluginPDMVersions);
    function GetAttachments: IPluginPDMVersions;
  end;

implementation

{$R *.dfm}

uses
  ListItemTilePainter,
  MiscGUI;


constructor TfrmFrameAttachments.Create(AOwner: TComponent;
  AAppProvider: TAppProvider);
begin
  inherited Create(AOwner);
  FAppProvider := AAppProvider;

  FAttachments := TPluginPDMVersionList.Create;

  FImagesCache := TImagesCache.Create(FAppProvider);
  lvAttachments.DoubleBuffered := True;
  lvAttachments.LargeImages := FImagesCache.ilLargeTypes;
end;

procedure TfrmFrameAttachments.CreateWnd;
begin
  inherited;
  lvAttachments.HandleNeeded;
  lvAttachments.ViewStyleEx := vsTile;
end;

procedure TfrmFrameAttachments.Clear;
begin
  lvAttachments.Items.BeginUpdate;
  try
    lvAttachments.Clear;
    FAttachments.Clear;
  finally
    lvAttachments.Items.EndUpdate;
    DoChange;
  end;
end;

procedure TfrmFrameAttachments.DeleteSelectedAttachments;
var
  i: Integer;
begin
  lvAttachments.Items.BeginUpdate;
  try
    for i := FAttachments.Count-1 downto 0 do
      if lvAttachments.Items[i].Selected then
      begin
        lvAttachments.Items.Delete(i);
        FAttachments.Delete(i);
      end;
    lvAttachments.Arrange(ComCtrls.arDefault);
  finally
    lvAttachments.Items.EndUpdate;
    DoChange;
  end;
end;

destructor TfrmFrameAttachments.Destroy;
begin
  FreeAndNil(FImagesCache);
  FreeAndNil(FAttachments);
  inherited;
end;

procedure TfrmFrameAttachments.DoChange;
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

function TfrmFrameAttachments.GetAttachment(Index: Integer): IPluginPDMVersion;
begin
  Result := FAttachments[Index];
end;

function TfrmFrameAttachments.GetAttachments: IPluginPDMVersions;
begin
  Result := CreatePluginPDMVersions(FAttachments);
end;

function TfrmFrameAttachments.GetCount: Integer;
begin
  Result := FAttachments.Count;
end;

procedure TfrmFrameAttachments.AAttachExecute(Sender: TObject);
begin
  //
end;

procedure TfrmFrameAttachments.lvAttachmentsAdvancedCustomDrawItem(
  Sender: TCustomListView; Item: TListItem; State: TCustomDrawState;
  Stage: TCustomDrawStage; var DefaultDraw: Boolean);

var
  Painter: TListItemTilePainter;
  Version: IPluginPDMVersion;

  procedure DrawFolderItem;
  //    
  begin
    Painter.DrawIcon;
    Painter.DrawText(FilteredGUID(Version.Product), '', Version.AccessLevel = alReadOnly);
  end;

  procedure DrawVersionItem;
  begin
    Painter.DrawIcon;
    Painter.DrawText(Version.Title, Version.TypeObj, Version.AccessLevel = alReadOnly);
  end;

begin
  case Stage of
    cdPostPaint:
        DefaultDraw := False;
    cdPrePaint:
      begin
        DefaultDraw := False;
        Painter := TListItemTilePainter.Create(Item);
        try
          Version := FAttachments[Item.Index];
          DrawVersionItem;
        finally
          FreeAndNil(Painter);
        end;
      end;
  end;
end;

procedure TfrmFrameAttachments.lvAttachmentsResize(Sender: TObject);
begin
  lvAttachments.Arrange(ComCtrls.arDefault);
  ShowScrollBar(lvAttachments.Handle, 0, false);
end;

procedure TfrmFrameAttachments.AddItem(AVersion: IPluginPDMVersion);
var
  ListItem: TListItem;
begin
  ListItem := lvAttachments.Items.Add;
  ListItem.Caption := AVersion.Title;
  ListItem.ImageIndex := FImagesCache.ilSmallTypes.GetIndex(AVersion.TypeObj);

  FAttachments.Add(AVersion);
end;

procedure TfrmFrameAttachments.Load(AVersions: IPluginPDMVersions);
var
  i: Integer;
begin
  if not Assigned(AVersions) then
    Exit;

  lvAttachments.Items.BeginUpdate;
  try
    for i := 0 to AVersions.Count-1 do
      if FAttachments.Find(AVersions.Version(i).IdVersion) = nil then
        AddItem(AVersions.Version(i));
  finally
    lvAttachments.Items.EndUpdate;
    DoChange;
  end;
end;

procedure TfrmFrameAttachments.ShowDlgAttach;
var
  Versions: IPluginPDMVersions;
begin
  if FAppProvider.GetAttachFromDlg(Versions) = mreOk then
    Load(Versions);
end;

end.
