unit PluginPDMVersionList;

interface

uses
  PluginIntf;

type
  TPDMVersionArray = array of IPluginPDMVersion;

  TPluginPDMVersionList = class
  private
    FVersions: TPDMVersionArray;
    function GetCount: Integer;
    function GetItem(Index: Integer): IPluginPDMVersion;
  public
    constructor Create;
    destructor Destroy; override;
    property Item[Index: Integer]: IPluginPDMVersion read GetItem; default;
    function Find(IdVersion: Integer): IPluginPDMVersion;
    procedure Add(Version: IPluginPDMVersion);
    procedure Delete(Index: Integer);
    procedure Clear;
    property Count: Integer read GetCount;
  end;


function CreatePluginPDMVersions(APDMVersions: TPluginPDMVersionList): IPluginPDMVersions;
function EqualsIds(AVersions1, AVersions2: IPluginPDMVersions): Boolean;


implementation

type
  TPluginPDMVersions = class(TInterfacedObject, IPluginPDMVersions)
  private
    FPDMVersions: TPDMVersionArray;
  private
    // IPluginPDMVersions
    function get_Count: Integer; safecall;
  public
    constructor Create(APDMVersions: TPDMVersionArray);
  public
    // IPluginPDMVersions
    property Count: Integer read get_Count;
    function Version(Index: Integer): IPluginPDMVersion; safecall;
  end;


function EqualsIds(AVersions1, AVersions2: IPluginPDMVersions): Boolean;
var
  i: Integer;
begin
  Result := True;
  if not Assigned(AVersions1) and not Assigned(AVersions2) then
    Exit;

  if Assigned(AVersions1) <> Assigned(AVersions2) then
  begin
    Result := False;
    Exit;
  end;

  if AVersions1.Count <> AVersions2.Count then
  begin
    Result := False;
    Exit;
  end;

  for i := 0 to AVersions2.Count - 1 do
    if AVersions1.Version(i).IdVersion <> AVersions2.Version(i).IdVersion then
    begin
      Result := False;
      Exit;
    end;
end;

function CreatePluginPDMVersions(APDMVersions: TPluginPDMVersionList): IPluginPDMVersions; overload;
var
  i: Integer;
  PDMVersionArray: TPDMVersionArray;
begin
  SetLength(PDMVersionArray, APDMVersions.Count);
  for i := 0 to APDMVersions.Count-1 do
  begin
    PDMVersionArray[i] := APDMVersions[i];
  end;

  Result := TPluginPDMVersions.Create(PDMVersionArray);
end;


{ TPluginPDMVersionList }

constructor TPluginPDMVersionList.Create;
begin
  Clear;
end;

destructor TPluginPDMVersionList.Destroy;
begin
  Clear;
  inherited;
end;

function TPluginPDMVersionList.Find(IdVersion: Integer): IPluginPDMVersion;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count-1 do
    if Item[i].IdVersion = IdVersion then
    begin
      Result := Item[i];
      Break;
    end;
end;

function TPluginPDMVersionList.GetCount: Integer;
begin
  Result := Length(FVersions);
end;

function TPluginPDMVersionList.GetItem(Index: Integer): IPluginPDMVersion;
begin
  Result := FVersions[Index];
end;

procedure TPluginPDMVersionList.Add(Version: IPluginPDMVersion);
begin
  if Find(Version.IdVersion) = nil then
  begin
    SetLength(FVersions, Length(FVersions) + 1);
    FVersions[High(FVersions)] := Version;
  end;
end;

procedure TPluginPDMVersionList.Delete(Index: Integer);
var
  i: Integer;
begin
  for i := Index to High(FVersions)-1 do
    FVersions[i] := FVersions[i+1];
  SetLength(FVersions, Length(FVersions)-1);
end;

procedure TPluginPDMVersionList.Clear;
begin
  SetLength(FVersions, 0);
end;


{ TPluginPDMVersions }

constructor TPluginPDMVersions.Create(APDMVersions: TPDMVersionArray);
begin
  FPDMVersions := APDMVersions;
end;

function TPluginPDMVersions.get_Count: Integer;
begin
  Result := Length(FPDMVersions);
end;

function TPluginPDMVersions.Version(Index: Integer): IPluginPDMVersion;
begin
  Result := FPDMVersions[Index];
end;

end.
